import 'dart:async';
import 'dart:io';

import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/dcoupon_listener.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/enrollment_form.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/user_accept_enrollment_form.dart';
import 'package:dcoupon_sdk_flutter_package/src/injection_container.dart';
import 'package:dcoupon_sdk_flutter_package/src/presentation/screens/enrollment_form/enrollment_form_screen.dart';
import 'package:dcoupon_sdk_flutter_package/src/themes/dark_theme.dart';
import 'package:dcoupon_sdk_flutter_package/src/themes/light_theme.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/translations.dart';
import 'package:flutter/material.dart';
import 'package:flutter_localizations/flutter_localizations.dart';

const int CLOSE_STATUS_CODE = 0;

class EnrollmentFormAppParams {
  final Translations translations;
  final Completer<DcouponListener> completer;
  final String jwt;
  final EnrollmentForm enrollmentForm;

  EnrollmentFormAppParams({
    required this.translations,
    required this.completer,
    required this.jwt,
    required this.enrollmentForm,
  });
}

class EnrollmentFormApp extends StatefulWidget {
  static const String routeName = '/enrollment';
  final EnrollmentFormAppParams enrollmentFormAppParams;

  const EnrollmentFormApp({
    Key? key,
    required this.enrollmentFormAppParams,
  }) : super(key: key);

  @override
  _EnrollmentFormAppState createState() => _EnrollmentFormAppState();
}

class _EnrollmentFormAppState extends State<EnrollmentFormApp> {
  void _closeEnrollment() {
    var dcouponListener = new DcouponListener();
    dcouponListener.code = CLOSE_STATUS_CODE;
    widget.enrollmentFormAppParams.completer.complete(dcouponListener);
    Navigator.of(context).pop();
  }

  void _acceptEnrollment() async {
    var dcouponListener = new DcouponListener();
    final userAcceptEnrollmentForm = sl<UserAcceptEnrollmentForm>();
    final failureOrAcceptEnrollmentFormResponse =
        await userAcceptEnrollmentForm(
      UserAcceptEnrollmentFormParams(
        jwt: widget.enrollmentFormAppParams.jwt,
        legalDocumentsAccepted: widget
            .enrollmentFormAppParams.enrollmentForm.legalDocumentsList
            .map((e) => e.id)
            .toList(),
      ),
    );

    failureOrAcceptEnrollmentFormResponse.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message =
              widget.enrollmentFormAppParams.translations.text('dcoupon_error');
          widget.enrollmentFormAppParams.completer.complete(dcouponListener);
          Navigator.of(context).pop();
        }
      },
      (acceptEnrollmentFormResponse) {
        final jwt = acceptEnrollmentFormResponse.sessionToken;
        final clientType = acceptEnrollmentFormResponse.clientType;

        ConfigReader.setUserJwt(jwt);
        ConfigReader.setClientType(clientType);
        
        dcouponListener.message = acceptEnrollmentFormResponse.sessionToken;
        dcouponListener.code = HttpStatus.ok;
        widget.enrollmentFormAppParams.completer.complete(dcouponListener);
        Navigator.of(context).pop();
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        return false;
      },
      child: MaterialApp(
        home: Theme(
          data: Theme.of(context).brightness == Brightness.dark
              ? dCouponDarkBlack()
              : dCouponLightWhite(),
          child: EnrollmentFormScreen(
            enrollmentForm: widget.enrollmentFormAppParams.enrollmentForm,
            onCloseEnrollment: _closeEnrollment,
            onAcceptEnrollment: _acceptEnrollment,
          ),
        ),
        theme: dCouponLightWhite(),
        darkTheme: dCouponDarkBlack(),
        themeMode: Theme.of(context).brightness == Brightness.dark
            ? ThemeMode.dark
            : ThemeMode.light,
        localizationsDelegates: [
          Translations.delegate,
          GlobalMaterialLocalizations.delegate,
          GlobalWidgetsLocalizations.delegate,
          GlobalCupertinoLocalizations.delegate
        ],
        supportedLocales: Translations.delegate.supportedLocales,
        debugShowCheckedModeBanner: false,
      ),
    );
  }
}
